@@ -94,6 +94,9 @@ test(hello2, Out == "Hello2 world2\nHello2 world2\nHello2 world2\nHello2 world2\
 test(hello3, Out == "Hello3 世界弐\n") :-
     hello3(世界弐, Out).
 
+test(hello4, Out == hello(world)) :-
+    hello4(Out).
+
 test(call_cpp, Out == "hello(foo)\n") :-
     with_output_to(string(Out), call_cpp(writeln(hello(foo)))).
 test(call_cpp, Out == "hello(世界四)\n") :-
@@ -133,7 +136,7 @@ test(as_string, S == "hello(ä¸\u0096ç\u0095\u008Cå\u009B\u009B)") :-
 test(add_3, Result == 666) :-
     add(667, -1, Result).
 test(add_3, Result == 123) :-
-    add(100.0, 23, Result).
+    add(100, 23, Result).
 test(add_3_err, error(type_error(integer,0.1))) :-
     add(666, 0.1, _).
 
@@ -813,17 +816,28 @@ test(blob) :-
     assertion(blob(Blob, my_blob)),
     close_my_blob(Blob).
 test(blob, error(my_blob_open_error(_),_)) :-
-    create_my_blob('FAIL', _).
+    create_my_blob('-FAIL_open-', _).
 test(blob, error(my_blob_close_error(Blob))) :-
-    create_my_blob('FAIL_close', Blob),
+    create_my_blob('-FAIL_close-', Blob),
     assertion(blob(Blob, my_blob)),
     close_my_blob(Blob).
+test(blob) :-
+    create_my_blob('foo', A),
+    with_output_to(string(Astr), write(current_output, A)),
+    assertion(string_concat("<my_blob>(", _, Astr)), % The pointer part is implementation-defined
+    free_blob(A),
+    with_output_to(string(Astr_freed), write(current_output, A)),
+    nil_repr(Nil),
+    format(string(Rstr), ">(~w)", [Nil]),
+    % The name part implementation-defined (e.g., mangled type name)
+    assertion(string_concat("<", _, Astr_freed)),
+    assertion(string_concat(_, Rstr, Astr_freed)).
 
 % The following attempts to test the handling of close errors in the
 % "release" callback, which calls ~MyBlob. It doesn't throw an error
 % (because it can't) but does output an error message.
 % You can run this test by hand:
-%    ?- create_fail_blob.
+%    ?- create_fail_close_blob.
 %    ?- force_gc.
 % and that should print:
 %    Close MyBlob failed: FAIL_close
@@ -831,11 +845,25 @@ test(blob, [blocked(cant_throw_error),
             error(e),
             setup(force_gc(GC_thread)),
             cleanup(restore_gc(GC_thread))]) :-
-    create_fail_blob,
+    create_fail_close_blob,
     garbage_collect,
     garbage_collect_atoms.
-create_fail_blob :-
-    create_my_blob('FAIL_close', Blob),
+test(blob, blocked('throws std::runtime_error')) :-
+    create_my_blob('-FAIL_connection-', _Blob).
+test(blob, error(my_blob_open_error(_))) :-
+    create_my_blob('-FAIL_open-', _Blob).
+test(blob, error(my_blob_fail_new(_))) :-
+    create_my_blob('-FAIL_new-', _Blob).
+test(blob, fail) :-
+    create_my_blob('-FAIL_compare-1', Blob1),
+    create_my_blob('-Fail_compare-2', Blob2),
+    Blob1 = Blob2.
+test(blob, error(my_blob_write_error(_))) :-
+    create_my_blob('-FAIL_write-', Blob),
+    with_output_to(string(_), write(current_output, Blob)).
+
+create_fail_close_blob :-
+    create_my_blob('-FAIL_close-', Blob),
     assertion(blob(Blob, my_blob)).
 
 test(blob, cleanup(close_my_blob(A))) :-
@@ -1029,6 +1057,31 @@ test_setup_call_cleanup(X) :-
         between(1, 5, X),
         throw(error)).
 
+% Experimental API tests
+test(plterm_scoped, R == []) :-
+    unify_atom_list([], R).
+test(plterm_scoped, R == [a, foo]) :-
+    unify_atom_list(["a", foo], R).
+test(plterm_scoped, error(type_error(list,foo))) :-
+    unify_atom_list(foo, _).
+
+test(plterm_scoped, R == []) :-
+    unify_atom_list_c([], R).
+test(plterm_scoped, R == [a, foo]) :-
+    unify_atom_list_c(["a", foo], R).
+test(plterm_scoped, error(type_error(list,foo))) :-
+    unify_atom_list_c(foo, _).
+
+test(plterm_scoped, [blocked('crashes in PL_free_term_ref')]) :-
+    term_release.
+
+test(record_ext, P == foo(bar,1,"a\0bc",'xy\0')) :-
+    record_ext(foo(bar,1,"a\0bc", 'xy\0'), Str),
+    record_ext(P, Str).
+test(record_ext, P == foo(bar,1,"a\0bc世界",'\0xy\0')) :-
+    record_ext2(foo(bar,1,"a\0bc世界", '\0xy\0'), Str),
+    record_ext2(P, Str).
+
 :- end_tests(cpp).
 
 :- begin_tests(cpp_atommap).
@@ -1055,7 +1108,7 @@ test(atom_term_map) :-
     % This test uses different keys from the other atom_term_map test.
     % If it succeeds, it will have erased all the entries that were
     % inserted.
-    atom_term_insert(foo, foo_value),
+    atom_term_insert(foo, "foo_value"),
     atom_term_insert(bar, bar_value),
     atom_term_insert(bar, bar_value), % OK to add identical enntry
     catch(atom_term_insert(foo, foo_value2),
@@ -1063,7 +1116,7 @@ test(atom_term_map) :-
           Exc = true),
     assertion(Exc == true),
     atom_term_find(foo, F),
-    assertion(F == foo_value),
+    assertion(F == "foo_value"),
     assertion(\+ atom_term_find(foox, _)),
     atom_term_erase(foo),
     assertion(\+ atom_term_find(foo, _)),
