@@ -3,7 +3,8 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2009-2017, VU University Amsterdam
+    Copyright (c)  2009-2024, VU University Amsterdam
+				 SWI-Prolog solutions b.v.
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -1501,18 +1502,23 @@ put_graph(turtle_state *ts, term_t g)
   if ( (cg=ts->current_graph) )
   { IOPOS *pos;
 
-    if ( !cg->v.r.handle )
-    { cg->v.r.handle = PL_new_atom_wchars(wcslen(cg->v.r.name), cg->v.r.name);
-    }
+    if ( !put_resource(ts, g, cg) )
+      return FALSE;
 
     if ( (pos=ts->input->position) )
-    { PL_put_variable(g);
-      return PL_unify_term(g, PL_FUNCTOR, FUNCTOR_colon2,
-				PL_ATOM, cg->v.r.handle,
-				PL_INT,  (int)pos->lineno);
+    { term_t line;
+      int rc;
+
+      rc = ( (line = PL_new_term_ref()) &&
+	     PL_put_int64(line, pos->lineno) &&
+	     PL_cons_functor(g, FUNCTOR_colon2, g, line) );
+
+      if ( rc )
+	PL_reset_term_refs(line);
+
+      return rc;
     } else
-    { return PL_put_atom(g, cg->v.r.handle);
-    }
+      return TRUE;
   } else
   { return TRUE;
   }
@@ -1960,7 +1966,7 @@ read_collection(turtle_state *ts)
 
 static resource *
 read_blank_node_property_list(turtle_state *ts)
-{ resource *olds, *bnode, *oldp = NULL;
+{ resource *olds = NULL, *bnode, *oldp = NULL;
   int rc;
 
   rc = ( set_anon_subject(ts, &olds) &&
@@ -2930,9 +2936,8 @@ retry:
 
       NO_GRAPH;
       if ( (r=read_blank_node_label(ts)) )
-      { set_subject(ts, r, NULL);
-	return final_predicate_object_list(ts);
-      }
+	return graph_or_final_predicate_object_list(ts, r,
+						    &graph_keyword_seen);
       return FALSE;
     }
     case ':':
