@@ -3,8 +3,9 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2007-2013, University of Amsterdam
+    Copyright (c)  2007-2024, University of Amsterdam
                               VU University Amsterdam
+                              SWI-Prolog solutions b.v.
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -36,8 +37,6 @@
 #ifndef PL_SEGSTACK_H_INCLUDED
 #define PL_SEGSTACK_H_INCLUDED
 
-#define SEGSTACK_CHUNKSIZE (1*1024)
-
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 The segchunk struct has its data attached   to  its back. The data field
 itself is typed double to ensure that   the  data is properly aligned to
@@ -60,6 +59,7 @@ typedef struct segchunk
 typedef struct
 { size_t   unit_size;
 					/* below clean using memset() */
+  int	   chunk_count;
   segchunk *first;
   segchunk *last;
   char	   *base;
@@ -74,9 +74,19 @@ emptySegStack(segstack *s)
 	 (s->last == NULL || s->last->previous == NULL);
 }
 
+/* Note that the C standard does not allow operations on NULL
+   pointers other than comparing equal to NULL.  For this reason
+   we need the `(stack)->top` condition.  I hope most C compiler
+   will delete the redundant condition.
+ */
+
+#define segStackHasData(stack)				\
+  ((stack)->top && (stack)->top > (stack)->base)
+#define segStackHasSpace(stack, size)				\
+  ((stack)->top && (stack)->top + (size) <= (stack)->max)
 
 #define popSegStack(stack, to, type) \
-	( ((stack)->top >= (stack)->base + sizeof(type))	\
+	( segStackHasData(stack)				\
 		? ( (stack)->top -= sizeof(type),		\
 		    *to = *(type*)(stack)->top,			\
 		    TRUE					\
@@ -86,21 +96,22 @@ emptySegStack(segstack *s)
 	)
 
 #define pushSegStack(stack, data, type) \
-	( ((stack)->top + sizeof(type) <= (stack)->max)	\
+	( segStackHasSpace(stack, sizeof(type))			\
 		? ( *(type*)(stack)->top = data,		\
 		    (stack)->top += sizeof(type),		\
 		    TRUE					\
 		  )						\
-		: pushSegStack_((stack), &data)			\
+		: !!pushSegStack_((stack), &data)		\
 	)
 
-COMMON(int)	pushSegStack_(segstack *stack, void* data) WUNUSED;
+COMMON(void*)	pushSegStack_(segstack *stack, void* data) WUNUSED;
 COMMON(int)	pushRecordSegStack(segstack *stack, Record r) WUNUSED;
 COMMON(int)	popSegStack_(segstack *stack, void *data);
 COMMON(void*)	topOfSegStack(segstack *stack);
 COMMON(void)	popTopOfSegStack_(segstack *stack);
 COMMON(void)	scanSegStack(segstack *s, void (*func)(void *cell));
 COMMON(void)	clearSegStack_(segstack *s);
+COMMON(void)	free_segstack_chunks(segchunk *c);
 
 		 /*******************************
 		 *	       INLINE		*
@@ -113,6 +124,19 @@ clearSegStack(segstack *s)
 }
 
 
+static inline void
+discardSegStack(segstack *s)
+{ segchunk *c = s->first;
+
+  if ( c )
+  { if ( !c->allocated )
+      c = c->next;
+    free_segstack_chunks(c);
+  }
+}
+
+
+
 static inline void
 topsOfSegStack(segstack *stack, int count, void **tops)
 { char *p = stack->top - stack->unit_size;
@@ -170,13 +194,14 @@ initSegStack(segstack *stack, size_t unit_size, size_t len, void *data)
     assert(len > sizeof(*chunk));
     assert(unit_size%sizeof(void*) == 0);
 #endif
-    chunk->size = len;
-    stack->base = stack->top = chunk->top = CHUNK_DATA(chunk);
-    stack->last = stack->first = chunk;
-    stack->max  = addPointer(chunk, len);
-    chunk->allocated = 0;
-    chunk->next      = NULL;
-    chunk->previous  = NULL;
+    chunk->size        = len;
+    stack->base        = stack->top = chunk->top = CHUNK_DATA(chunk);
+    stack->last        = stack->first = chunk;
+    stack->max         = addPointer(chunk, len);
+    stack->chunk_count = 1;
+    chunk->allocated   = 0;
+    chunk->next        = NULL;
+    chunk->previous    = NULL;
   } else
   { memset(&stack->first, 0, sizeof(*stack)-offsetof(segstack,first));
   }
